'use client'
import { createIbcRequest, deleteIbc, editIbcRequest } from "@/app/lib/ibc/action";
import DataTable from "@/components/tables/DataTable";
import { DeleteIcon, EditIcon } from "@chakra-ui/icons";
import { Button, Flex, Icon, useDisclosure, useToast } from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { useState } from "react";
import { columnsDataBlocks } from "../variables/columnsBlock";
import IbcModalForm from "./form";
import { IbcInterface } from "@/types/settings/ibc/ibc";
import { hasPermission } from "@/utils/navigation";
import { useGeneral } from "@/contexts/GeneralContext";

interface IbcTableProps {
  data: { data: IbcInterface[]; }
}

const IbcTable: React.FC<IbcTableProps> = ({ data }) => {
  const { currentUser } = useGeneral();
  const { isOpen, onOpen, onClose } = useDisclosure();
  const [selectedIbc, setSelectedIbc] = useState<IbcInterface | null>(null);
  const toast = useToast();
  const router = useRouter();

  const deleteUser = async (id: number) => {
    const res = await deleteIbc(id);
    if (res?.status == 200) {
      toast({
        title: "Success",
        description: res.message,
        status: "success",
        position: "top-right",
        duration: 2000,
        isClosable: true,
      });
      router.refresh();
    } else {
      toast({
        title: "Error",
        description: res.message,
        status: "error",
        position: "top-right",
        duration: 9000,
        isClosable: true,
      });
    }
  };

  const handleAddClick = () => {
    setSelectedIbc(null); // Reset for add
    onOpen();
  };

  const handleEditClick = (user: IbcInterface) => {
    const singleUser = data?.data?.find((item: any) => (item.id == user));
    setSelectedIbc((prev) => {
      return { ...singleUser };
    });
    onOpen();
  };

  const handleSubmit = async (values: any) => {
    if (selectedIbc) {
      const payload = { userId: selectedIbc.id, data: values };
      const res = await editIbcRequest(payload);
      if (res?.status == 200) {
        toast({
          title: "Success",
          description: res.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
      } else {
        toast({
          title: "Error",
          description: res.message,
          status: "error",
          position: "top-right",
          duration: 9000,
          isClosable: true,
        });
      }
    } else {
      const res: any = await createIbcRequest(values);
      if (res?.status == 200) {
        toast({
          title: "Success",
          description: res.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
      } else {
        toast({
          title: "Error",
          description: res.message,
          status: "error",
          position: "top-right",
          duration: 9000,
          isClosable: true,
        });
      }
    }
  };

  const action = (user: any) => (
    <Flex gap={2}>
      {hasPermission(currentUser.permissions, "setting.ibc", "edit") && (
        <Button
          variant="blank"
          onClick={() => handleEditClick(user)}
          w="37px"
          h="37px"
          borderRadius="10px">
          <Icon as={EditIcon} color="brand.500" w="20px" h="20px" />
        </Button>
      )}
      {hasPermission(currentUser.permissions, "setting.ibc", "delete") && (
        <Button
          variant="blank"
          onClick={() => deleteUser(user)}
          w="37px"
          h="37px"
          borderRadius="10px">
          <Icon as={DeleteIcon} color="brand.500" w="20px" h="20px" />
        </Button>
      )}
    </Flex>
  );

  return (
    <>
      <DataTable
        addButton={hasPermission(currentUser.permissions, "setting.ibc", "create") && <Button variant="solid" colorScheme="blue" onClick={handleAddClick}>Create</Button>}
        columnsData={columnsDataBlocks}
        tableData={data?.data as unknown as IbcInterface[]}
        paginate={data}
        isSelectable={false}
        showPageSize={false}
        actionSlot={(user: any) => action(user)}
      />

      <IbcModalForm
        isOpen={isOpen}
        onClose={onClose}
        currentUser={selectedIbc}
        handleSubmit={handleSubmit}
      />
    </>
  );
};

export default IbcTable;
